
*** Summary statistics table
*** BY: David Qihang Wu
*** Inputs: Matthew Pecenco

****************************************************************
capture program drop sumstats
program define sumstats
****************************************************************
	/* What this program does is:
	   - Generate the mean and sd for group 1 (could be control group) and group 2 (could be treatment group), potentially many subgroups within each category 
	   - Between control and treatment, for arbitrary pair, do differences
	   
	   Structure when doing  descriptive statistics:
	   - Group 1/2:  specified in gp1/gp2, titled with var label, can specify the general title for this subgroup in gp1name/gp2name
	     ** Please make sure they are 0-1 variables
	   - Difference: specified in diffexplicitly, should follow the format "var1-var2 var3-var4"
	     ** To do difference of specified pair, I would only keep observations in either group, and then compare means 
		 ** You can specify cluster and FE for differencing
	   - diffsilently: A specific type of differencing, can be only applied when diffexplicitly not specified; 
			diffpair is specified as "treatment-control treatment-control", treatment/control must come from gp1/gp2
	     ** What it does: treat gp1 as control, then compare each subgroup of gp2 to control; only indicate significance of difference next to gp2 mean
	   - If none of the groups above specified, I'll just summarize variables for all observations.
	   		
	   Formatting:
	   - npanel, panellabel: to add panel titles, you need to specify how many variables in one panel first (npanel), and then specify the title (panellabel)
			then I'll break down varlist (in order) by this numlist
	   - noobs: by default, the first column would generate count for non-missing obs for each var. Specify this and this column is gone.
	   - nosd: by default, show the standard deviation (error)
	   - sdwide: Put standard deviation right next to mean, not below
	   - nofirsttitle: by default, the first title of the table is "Obs/Mean outcome".
	   - firstrowobs: The first row of the table shows the total number of each group. This is simply to put a label to this row. 
	   - nonumber: No numbers in the title
	*/

	syntax varlist, filename(string) [gp1(varlist) gp2(varlist)] [gp1name(string) gp2name(string)] ///
					[fe(varlist)] [cluster(varlist)] [diffexplicitly(string)] [diffsilently(string)] ///
					[npanel(numlist integer) panellabel(string)] ///
					[nosd] [sdwide] [noobs] [firstrowobs(string)] ///
					[nofirsttitle] [nonumber] 

	****************** __init__ *********************
	
	** Read important dimensions
	local n_var = wordcount("`varlist'")
	local n_gp1 = wordcount("`gp1'")
	local n_gp2 = wordcount("`gp2'")
	
	local diffexplicitly = ustrtrim(stritrim("`diffexplicitly'"))
	if "`diffexplicitly'"==""{
		local n_diff = 0
	}
	else{
		local len = strlen("`diffexplicitly'")
		local n_diff = 1
		forvalues i=1/`len'{
			if substr("`diffexplicitly'",`i',1)==" "{
				local n_diff = `n_diff'+1
			}
		}
	}

	** Particular for command "diffsilently"
	local silent = 1
	if "`diffsilently'"!=""{
		if ("`diffexplicitly'"==""){
			local silent = 0
			local len = strlen("`diffsilently'")
			local n_diff = 1
			forvalues i=1/`len'{
				if substr("`diffsilently'",`i',1)==" "{
					local n_diff = `n_diff'+1
				}
			}
			local diffexplicitly = "`diffsilently'"
		}
		else{
			display as error "You can't specify diffexplicitly"
			exit
		}
	}
	
	** See if "nosd" and "sdwide" are reported together
	if "`sd'"!="" & "`sdwide'"!=""{
		display as error "You can't specify nosd an sdwide together"
		exit
	}
	
	** Define main stata matrix
	if `n_gp1'!=0 | `n_gp2'!=0{
		matrix matmean = J(`n_var'+1,1+`n_gp1'+`n_gp2'+`n_diff', .)
		matrix matsd = J(`n_var'+1,1+`n_gp1'+`n_gp2'+`n_diff', .)
		matrix matpvalue = J(`n_var'+1,1+`n_gp1'+`n_gp2'+`n_diff', .)
	}
	else{
		matrix matmean = J(`n_var'+1,2, .)
		matrix matsd = J(`n_var'+1,2, .)
		matrix matpvalue = J(`n_var'+1,2, .)
	}
	
	****************** __main__ *********************

	** Construct main stata matrix 
	** Step 1: If no specified gp1 or gp2 - just summarize all
	** Step 2: For gp1 and gp2, summarize within each group
	** Step 3: Do differences for pairs specified in diffexplicitly
	forvalues i = 1/`n_var' {
		local var: word `i' of `varlist'
		
		* To calculate how many non-missing values:
		tempvar dv555nonmissing
		gen `dv555nonmissing' = (`var'!=.)
		
		matrix matmean[`i',1] = .
		
		* Step 1
		if `n_gp1'==0 & `n_gp2'==0{
			quietly sum `var' if `dv555nonmissing'==1
			matrix matmean[`i',1] = r(N)
			matrix matmean[`i',2] = r(mean)
			matrix matsd[`i',2] = r(sd)		
		}
		
		* Step 2
		** first, get the mean and sd of gp1 and gp2
		if `n_gp1'>0{
		local countobs = 0
		forvalues j = 1/`n_gp1'{
			local groupvar: word `j' of `gp1'
			
			** if groupvar is not 0-1 variable, report
			preserve
				drop if `groupvar'==.
				egen isit01 = anymatch(`groupvar'), values(0 1)
				quietly tab isit01
				if r(r)==2{
					display as error "`groupvar' is not a 0-1 variable"
					exit
				}
			restore
			
			gen group1 = `groupvar'
			quietly sum `var' if group1==1 & `dv555nonmissing'==1
			matrix matmean[`i',`j'+1] = r(mean)
			matrix matsd[`i',`j'+1] = r(sd)	
			quietly sum `var' if (group1==1 | group1==0) & `dv555nonmissing'==1
			local countobs = max(`countobs', r(N))
			drop group1
		}
		matrix matmean[`i',1] = min(matmean[`i',1],`countobs') // This is at least one variable within gp1 is equal to 1
		}
		
		if `n_gp2'>0{
		local countobs = 0
		forvalues j = 1/`n_gp2'{
			local groupvar: word `j' of `gp2'
			
			** if groupvar is not 0-1 variable, report
			preserve
				drop if `groupvar'==.
				egen isit01 = anymatch(`groupvar'), values(0 1)
				quietly tab isit01
				if r(r)==2{
					display as error "`groupvar' is not a 0-1 variable"
					exit
				}
			restore

			gen group2 = `groupvar'
			quietly sum `var' if group2==1 & `dv555nonmissing'==1
			matrix matmean[`i',`j'+`n_gp1'+1] = r(mean)
			matrix matsd[`i',`j'+`n_gp1'+1] = r(sd)	
			quietly sum `var' if (group2==1 | group2==0) & `dv555nonmissing'==1
			local countobs = max(`countobs',r(N))
			drop group2
		}
		matrix matmean[`i',1] = min(matmean[`i',1],`countobs') // This is at least one variable within gp2 is equal to 1
		}
		
		* Step 3: Difference for specific pairs
		if `n_diff'>0{
			local diffexplicitly_tmp = "`diffexplicitly'"+" "
			forvalues k = 1/`n_diff'{
				local tmp = substr("`diffexplicitly_tmp'",1,strpos("`diffexplicitly_tmp'"," ")-1)
				local diffexplicitly_tmp = substr("`diffexplicitly_tmp'", strpos("`diffexplicitly_tmp'"," ")+1,.)
				local treatment = substr("`tmp'",1,strpos("`tmp'","-")-1)
				local control = substr("`tmp'",strpos("`tmp'","-")+1,.)
				preserve
					keep if `control'==1 | `treatment'==1
					if "`fe'"!="" & "`cluster'"==""{
						quietly reghdfe `var' `treatment' if `dv555nonmissing'==1, absorb(`fe')
					}
					if "`fe'"=="" & "`cluster'"!=""{
						quietly reghdfe `var' `treatment' if `dv555nonmissing'==1, vce(cluster `cluster')
					}
					if "`fe'"!="" & "`cluster'"!=""{
						quietly reghdfe `var' `treatment' if `dv555nonmissing'==1, absorb(`fe') vce(cluster `cluster')
					}
					if "`fe'"=="" & "`cluster'"==""{
						quietly reg `var' `treatment' if `dv555nonmissing'==1
					}
					matrix matmean[`i',`n_gp1'+`n_gp2'+`k'+1] = _b[`treatment']
					matrix matsd[`i',`n_gp1'+`n_gp2'+`k'+1] = _se[`treatment']
					matrix matpvalue[`i',`n_gp1'+`n_gp2'+`k'+1] = (2 * ttail(e(df_r), abs(_b[`treatment']/_se[`treatment']))) 
				restore
			}	
		}
	}
	
	* Add number of observations of each group
	if `n_gp1'==0 & `n_gp2'==0{
		matrix matmean[`n_var'+1, 2] = _N
	}
	forvalues j = 1/`n_gp1'{
		local groupvar: word `j' of `gp1'
		quietly count if `groupvar'==1
		matrix matmean[`n_var'+1, `j'+1] = r(N)
	}
	
	forvalues j = 1/`n_gp2'{
		local groupvar: word `j' of `gp2'
		quietly count if `groupvar'==1
		matrix matmean[`n_var'+1, `j'+`n_gp1'+1] = r(N)
	}
		
	****************** Write into Mata *********************
	** More initiation
	local obscol = 1
	if "`obs'"!=""{
		local obscol = 0
	}
	local sdbelow_yesno = 1
	if "`sd'"!=""{
		local sdbelow_yesno = 0
	}
	local firstrow = 0
	if "`firstrowobs'"!=""{
		local firstrow = 1
	}
	local sdnext_yesno = 0
	if "`sdwide'"!=""{
		local sdnext_yesno = 1
		local sdbelow_yesno = 0
	}
	
	** Define mata matrix
	if `n_gp1'==0 & `n_gp2'==0{
		mata: grandmat = J(`n_var'*(1+`sdbelow_yesno')+`firstrow',1+`obscol'+(1              +`n_diff'*`silent')*(1+`sdnext_yesno'), "")
	}
	else{
		mata: grandmat = J(`n_var'*(1+`sdbelow_yesno')+`firstrow',1+`obscol'+(`n_gp1'+`n_gp2'+`n_diff'*`silent')*(1+`sdnext_yesno'), "")
		***                                                     label  obs     gp1     gp2        diff               if sdwide 
	}
	
	mata matmean = st_matrix("matmean")
	mata matsd = st_matrix("matsd")
	mata matpvalue = st_matrix("matpvalue")
	
	** Store label names in the first column
	forvalues i = 1/`n_var'{                       
		local name: word `i' of `varlist'
		local label: var label `name'
		mata: i=strtoreal(st_local("i"))
		mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1] = st_local("label")
	}
	
	** First row
	if `firstrow'==1{
		mata: grandmat[`firstrow',1] = "`firstrowobs'"
	}
	mata: i = strtoreal(st_local("n_var"))  
	local totalcol = (max(`n_gp1'+`n_gp2',1)+`n_diff'*`silent')*(1+`sdnext_yesno')
	local tmp = max(`n_gp1'+`n_gp2',1)
	if "`firstrowobs'"!=""{
		forvalues j = 1/`tmp'{
			mata: j = strtoreal(st_local("j"))
			mata: grandmat[1, 1+`obscol'+(j-1)*(1+`sdnext_yesno')+1] = strofreal(matmean[i+1,j+1], "%3.0f")
		}
	}
			
	** Store mean and diff
	if `silent'==1{
	forvalues i = 1/`n_var'{                       
		mata: i = strtoreal(st_local("i"))
		local tmp = max(`n_gp1'+`n_gp2',1)
		if `obscol'>0{
			mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1+`obscol'] = strofreal(matmean[i,1],"%3.0f")
		}
		forvalues j = 1/`tmp'{
			mata: j = strtoreal(st_local("j"))
			mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1+`obscol'+(j-1)*(1+`sdnext_yesno')+1] = strofreal(matmean[i,1+j], "%3.2f")
			if `sdbelow_yesno'==1{
				mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+2,1+`obscol'+(j-1)*(1+`sdnext_yesno')+1] = "("+strofreal(matsd[i,1+j], "%3.2f")+")"
			}
			if `sdnext_yesno'==1{
				mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1+`obscol'+(j-1)*(1+`sdnext_yesno')+2] = "("+strofreal(matsd[i,1+j], "%3.2f")+")"
			}			
		}
		local tmp2 = `tmp'+`n_diff'
		local tmp1 = `tmp'+1
		forvalues j = `tmp1'/`tmp2'{
			mata: j = strtoreal(st_local("j"))
			mata: star=""
			mata: if (matpvalue[i,1+j]<0.01) star="";;
			mata: if (matpvalue[i,1+j]>=0.01 && matpvalue[i,1+j]<0.05) star="";;
			mata: if (matpvalue[i,1+j]>=0.05 && matpvalue[i,1+j]<0.10) star="";;
			mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1+`obscol'+(j-1)*(1+`sdnext_yesno')+1] = strofreal(matpvalue[i,1+j], "%3.2f")+star
			if `sdbelow_yesno'==1{
				mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+2,1+`obscol'+(j-1)*(1+`sdnext_yesno')+1] = "("+strofreal(matsd[i,1+j], "%3.2f")+")"
			}
			if `sdnext_yesno'==1{
				mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1+`obscol'+(j-1)*(1+`sdnext_yesno')+2] = "("+strofreal(matsd[i,1+j], "%3.2f")+")"
			}			
		}
	}
	}
	else{
	forvalues i = 1/`n_var'{
		mata: i = strtoreal(st_local("i"))
		local tmp = max(`n_gp1'+`n_gp2',1)
		if `obscol'>0{
			mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1+`obscol'] = strofreal(matmean[i,1],"%3.0f")
		}
		forvalues j = 1/`tmp'{
			mata: j = strtoreal(st_local("j"))

			* See if this variable appears in one of the treatment variables in diffsilently
			if `j'<=`n_gp1'{
				local var: word `j' of `gp1'
			}
			else{
				local jtmp = `j'-`n_gp1'
				local var: word `jtmp' of `gp2'
			}
			
			local diffexplicitly_tmp = "`diffexplicitly'"+" "
			local tag=0
			mata: star=""
			forvalues k=1/`n_diff'{
			if `tag'==0{
				local strtmp = substr("`diffexplicitly_tmp'",1,strpos("`diffexplicitly_tmp'"," ")-1)
				local diffexplicitly_tmp = substr("`diffexplicitly_tmp'", strpos("`diffexplicitly_tmp'"," ")+1,.)
				local treatment = substr("`strtmp'",1,strpos("`strtmp'","-")-1)
				local control = substr("`strtmp'",strpos("`strtmp'","-")+1,.)
				if "`treatment'"=="`var'"{
					local kk = `k'+`tmp'
					mata: kk = strtoreal(st_local("kk"))
					mata: if (matpvalue[i,1+kk]<0.01) star="";;
					mata: if (matpvalue[i,1+kk]>=0.01 && matpvalue[i,1+kk]<0.05) star="";;
					mata: if (matpvalue[i,1+kk]>=0.05 && matpvalue[i,1+kk]<0.10) star="";;
					local tag=1
				}
			}
			}
			mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1+`obscol'+(j-1)*(1+`sdnext_yesno')+1] = strofreal(matpvalue[i,1+j], "%3.2f")+star
			if `sdbelow_yesno'==1{
				mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+2,1+`obscol'+(j-1)*(1+`sdnext_yesno')+1] = "("+strofreal(matsd[i,1+j], "%3.2f")+")"
			}
			if `sdnext_yesno'==1{
				mata: grandmat[`firstrow'+(1+`sdbelow_yesno')*(i-1)+1,1+`obscol'+(j-1)*(1+`sdnext_yesno')+2] = "("+strofreal(matsd[i,1+j], "%3.2f")+")"
			}			
		}
	}
	}
	
	
	****************** Format in LaTex *********************
	** Label columns
	if `obscol'>0{
		local collabel "& " 
	}
	if `n_gp1'==0 & `n_gp2'==0{
		local collabel = "&\multicolumn{" + string(`sdnext_yesno'+1) + "}{c}{All sample}"
	}
	forvalues j = 1/`n_gp1'{
		local name: word `j' of `gp1'
		local label: var label `name'
		local tmp = "&\multicolumn{" + string(`sdnext_yesno'+1) + "}{c}{" + "`label'" + "} "
		local collabel `collabel' `tmp'
	}
	
	forvalues j = 1/`n_gp2'{
		local name: word `j' of `gp2'
		local label: var label `name'
		local tmp = "&\multicolumn{" + string(`sdnext_yesno'+1) + "}{c}{" + "`label'" + "} "
		local collabel `collabel' `tmp'
	}
	if `n_diff'*`silent'>0{
		local diffexplicitly_tmp = "`diffexplicitly'"+" "
		forvalues k = 1/`n_diff'{
			local tmp = substr("`diffexplicitly_tmp'",1,strpos("`diffexplicitly_tmp'"," ")-1)
			local diffexplicitly_tmp = substr("`diffexplicitly_tmp'", strpos("`diffexplicitly_tmp'"," ")+1,.)
			local treatment = substr("`tmp'",1,strpos("`tmp'","-")-1)
			local control = substr("`tmp'",strpos("`tmp'","-")+1,.)
			local label_treat: var label `treatment'
			local label_con: var label `control'
			local tmp = "&\multicolumn{" + string(`sdnext_yesno'+1)+ "}{c}{" + substr("`label_treat'",1,5)+"-"+substr("`label_con'",1,5) + "} "
			local collabel `collabel' `tmp'
		}
	}
			
	** Insert panel label
	local i=`firstrow'+1
	di "`panellabel'"
	local tmplabel = "`panellabel'"
	local totalcol = `obscol'+(`n_gp1'+`n_gp2'+`n_diff'*`silent')*(`sdnext_yesno'+1)+1
	if `n_gp1'==0 & `n_gp2'==0{
		local totalcol = `obscol'+1*(1+`sdnext_yesno')+1
	}
	if !missing(`n_panel'){
		local n_panel = wordcount("`npanel'")
	}
	else{
		local n_panel=0
	}

	if `n_panel'>0{
	foreach num of numlist `npanel'{
		if `i'==1{
			local tmpstr = "\multicolumn{`totalcol'}{l}{\textit{"+trim(substr("`tmplabel'",1,strpos("`tmplabel'",";")-1))+"}} \\"
		}
		else{
			local tmpstr = "\\ \multicolumn{`totalcol'}{l}{\textit{"+trim(substr("`tmplabel'",1,strpos("`tmplabel'",";")-1))+"}} \\"
		}
		local tmpstr = `""`tmpstr'""'
		local tmplabel = trim(substr("`tmplabel'",strpos("`tmplabel'",";")+1,.))
		local panelstring `panelstring' `i' `tmpstr'
		local i = real("`i'")+(`sdbelow_yesno'+1)*real("`num'")
	}
	}
	
	** Locate gp1, gp2, diff
	local start_gp1 = 1+`obscol'+1
	local end_gp1 = 1+`obscol'+`n_gp1'*(`sdnext_yesno'+1)
	local start_gp2 = 1+`obscol'+`n_gp1'*(`sdnext_yesno'+1)+1
	local end_gp2 =   1+`obscol'+(`n_gp1'+`n_gp2')*(`sdnext_yesno'+1)
	local start_diff = 1+`obscol'+(`n_gp1'+`n_gp2')*(`sdnext_yesno'+1)+1
	if `n_gp1'==0 & `n_gp2'==0{
		local start_diff = 1+`obscol'+(`sdnext_yesno'+1)*1+1
	}

	** Numbers
	forvalues i=2/`totalcol'{
		local tmp = "&\multicolumn{1}{c}{("+string(`i'-1)+")} "
		local multicol `multicol' `tmp'
		local preprehead = "`preprehead'"+"c"
	}
	if "`number'"==""{
		local tmp = "\cmidrule(lr){2-`totalcol'}"
		local multicol `tmp' `multicol'
		local tmp = "\\"
		local multicol `multicol' `tmp'
	}
	else{
		local multicol ""
	}

	** First title
	local i = `obscol'+1
	if "`firsttitle'"==""{
		if "`obs'"==""{
			local prehead "& Obs"
		}
		else{
			local prehead ""
		}
		
		while `i' <= `totalcol'-1{
			local tmp = "&"
			if ((`n_gp1'+`n_gp2')*(`sdnext_yesno'+1)+1)/2==`i'-1{
				local tmp = "`tmp'"+" Mean outcomes"
			}
			if ((`n_gp1'+`n_gp2')*(`sdnext_yesno'+1)+1)/2==`i'-0.5{
				local tmp = "`tmp'"+"\multicolumn{2}{c}{Mean outcomes}"
				local ++i
			}
			if `silent'==1{
			if ((`n_gp1'+`n_gp2')*(`sdnext_yesno'+1)+(`n_diff'*(`sdnext_yesno'+1)+1)/2==`i'-1){
				local tmp = "`tmp'"+" Difference"
			}
			if ((`n_gp1'+`n_gp2')*(`sdnext_yesno'+1)+(`n_diff'*(`sdnext_yesno'+1)+1)/2==`i'-0.5) & (`n_diff'>0){
				local tmp = "`tmp'"+"\multicolumn{2}{c}{Difference}"
				local ++i
			}
			}
			local prehead `prehead' `tmp'
			local ++i
		}

		
		if `n_gp1'==0 & `n_gp2'==0{
			if "`obs'"==""{
				local prehead = "& Obs &\multicolumn{" + string(`sdnext_yesno'+1) + "}{c}{Mean outcomes}"
			}
			else{
				local prehead = "&\multicolumn{" + string(`sdnext_yesno'+1) + "}{c}{Mean outcomes}"
			}
		}
		local tmp = "\\"
		local prehead `prehead' `tmp'
	}
		
	** Additional header
	if `obscol'>0{
		local tmpline "& " 
	}
	if (`n_gp1'>0) & "`gp1name'"!=""{
		local tmp = "&\multicolumn{" +string(`n_gp1'*(`sdnext_yesno'+1)) +"}{c}{`gp1name'} "
		local tmpline `tmpline' `tmp'
		local tmpinsert_gp1 = "\cmidrule(lr){`start_gp1'-`end_gp1'}"
	}
	else{
		local sidetmp = `n_gp1'*(`sdnext_yesno'+1)
		forvalues i=1/`sidetmp'{
			local tmp "&"
			local tmpline `tmpline' `tmp'
		}
	}

	if (`n_gp2'>0) & "`gp2name'"!=""{
		local tmp = "&\multicolumn{" +string(`n_gp2'*(`sdnext_yesno'+1)) +"}{c}{`gp2name'} "
		local tmpline `tmpline' `tmp'
		local tmpinsert_gp2 = "\cmidrule(lr){`start_gp2'-`end_gp2'}"
	}
	else{
		local sidetmp = `n_gp2'*(`sdnext_yesno'+1)
		forvalues i=1/`sidetmp'{
			local tmp "&"
			local tmpline `tmpline' `tmp'
		}
	}

	local difftmp = `n_diff'*(`sdnext_yesno'+1)*`silent'
	forvalues i=1/`difftmp'{
		local tmp = "&" 
		local tmpline `tmpline' `tmp'
	}
	
	if "`gp1name'"=="" & "`gp2name'"==""{
		local tmpline ""
	}
	else{
		local tmp = "\\"
		local tmpline `tmpline' `tmp'
	}	

	** cmidrule
	if "`firsttitle'"==""{
	if `obscol'==1{
		local cmidrule "\cmidrule(lr){2-2}"
	}
	else{
		local cmidrule ""
	}
	if `n_gp1'==0 & `n_gp2'==0{
		local cmidrule "`cmidrule'" "\cmidrule(lr){`start_gp1'-`start_gp1'}"
	}
	if `n_gp1'>0 & `n_gp2'==0{
		local cmidrule "`cmidrule'" "\cmidrule(lr){`start_gp1'-`end_gp1'}"
	}
	if `n_gp2'>0{
		local cmidrule "`cmidrule'" "\cmidrule(lr){`start_gp1'-`end_gp2'}"
	}
	if `n_diff'*`silent'>0{
		local cmidrule "`cmidrule'" "\cmidrule(lr){`start_diff'-`totalcol'}"
	}
	}
		
	#delimit ;
	local header "\def\sym#1{\ifmmode^{#1}\else\(^{#1}\)\fi} 
				  \begin{tabular}{l*{1}{`preprehead'}}  
				  \toprule `prehead' `cmidrule'
				  `tmpline'
				  `tmpinsert_gp1' `tmpinsert_gp2' `collabel' \\ 
				  `multicol'
				  \midrule" ;
	local endtex "\bottomrule \end{tabular}";
	mmat2tex grandmat using "`filename'.tex", replace preheader("`header'") ///
			bottom("`endtex'") rownames(`rowlabel') 
			insertendrow(`panelstring')
	;
	#delimit cr
	
end
